<!DOCTYPE html>
<html lang="{{ site.active_lang }}" dir="{% if site.active_lang == 'ar' or site.active_lang == 'he' %}rtl{% else %}ltr{% endif %}">

  {% include head.html %}

  <body theme-base-08>

    {% include sidebar.html %}

    <div class="content container">
      {{ content }}
    </div>

  </body>
  <script async src="https://www.googletagmanager.com/gtag/js?id={{ site.ga_tag }}"></script>
  <script>
    window.dataLayer = window.dataLayer || [];
    function gtag(){dataLayer.push(arguments);}
    gtag('js', new Date());
  
    gtag('config', '{{ site.ga_tag }}');
  </script>
</html>
