<!-- jekyll-redirect-from will use this template to implement redirects. This modification to the template helps polyglot redirect accounting for languages -->
{% assign redirection = page.redirect.to | remove_first: site.url %}
{% if site.active_lang == site.default_lang %}
{% assign redirectTo =  site.url | append: redirection %}
{% else %}
{% assign redirectTo = site.url | append: "/" | append: site.active_lang | append: redirection  %}
{% endif %}
<!DOCTYPE html>
<html lang="en-US">
  <meta charset="utf-8">
  <title>Redirecting&hellip;</title>
  <link rel="canonical" {% static_href %}href="{{ redirectTo }}"{% endstatic_href %}>
  <script>location="{{ redirectTo }}" + window.location.hash</script>
  <meta http-equiv="refresh" content="0; url={{ redirectTo }}">
  <meta name="robots" content="noindex">
  <h1>Redirecting&hellip;</h1>
  <a {% static_href %}href="{{ redirectTo }}"{% endstatic_href %}>Click here if you are not redirected.</a>
</html>
